package be.iminds.ilabt.jfed.experimenter_gui.slice.actions;

import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentViewController;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.experimenter_gui.util.FileUtils;
import be.iminds.ilabt.jfed.highlevel.util.ESpecMetaFileCreator;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.rspec.model.LoginService;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecInterface;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.stage.FileChooser;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 16:32
 */
public class SaveExperimentInfoJsonAction extends ExperimentViewAction {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(SaveExperimentInfoJsonAction.class);

    private final TaskService taskService;
    private final JFedPreferences jFedPreferences;
    private final GeniUserProvider geniUserProvider;

    SaveExperimentInfoJsonAction(ExperimentViewController experimentViewController, TaskService taskService, JFedPreferences jFedPreferences, GeniUserProvider geniUserProvider) {
        super(experimentViewController);
        this.taskService = taskService;
        this.jFedPreferences = jFedPreferences;
        this.geniUserProvider = geniUserProvider;
    }


    @Override
    public void run() {
        assert (Platform.isFxApplicationThread());

        FileChooser fc = new FileChooser();
        fc.setTitle("Save experiment-info.json");
        fc.getExtensionFilters().addAll(FileUtils.JSON_EXTENSION_FILTER, FileUtils.ALL_EXTENSION_FILTER);
        fc.setSelectedExtensionFilter(FileUtils.JSON_EXTENSION_FILTER);


        fc.setInitialFileName(experiment.getSliceOrNull().getName() + "-experiment-info" + FileUtils.JSON_FILE_EXTENSION);


        //set the default save dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));

        if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST)) {
            File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST));
            if (dir.exists()) {
                fc.setInitialDirectory(dir);
            }
        }

        File file = fc.showSaveDialog(parentWindow);

        if (file == null)
            return;

        Task<Void> saveTask;

        //first make sure that the file-extension is present
        if (!file.getAbsolutePath().endsWith(FileUtils.JSON_FILE_EXTENSION))
            file = new File(file.getAbsolutePath() + FileUtils.JSON_FILE_EXTENSION);

        saveTask = new SaveExperimentInfoJsonTask(file);

        taskService.submitTask(saveTask);
    }

    public class SaveExperimentInfoJsonTask extends Task<Void> {

        private final File file;

        public SaveExperimentInfoJsonTask(File file) {
            this.file = file;
        }

        @Override
        protected Void call() throws Exception {
            LOG.debug("Saving experiment info JSON in \"" + file.getAbsolutePath() + "\"");
            updateMessage("Saving experiment info JSON in \"" + file.getAbsolutePath() + "\"");

            byte[] fileContent;
            try {
                ESpecMetaFileCreator eSpecMetaFileCreator = new ESpecMetaFileCreator(experiment, geniUserProvider.getLoggedInGeniUser());
                fileContent = eSpecMetaFileCreator.getExperimentInfoJson();
            } catch (final Exception ex) {
                LOG.error("Error while generating experiment info JSON", ex);
                Platform.runLater(() -> JFDialogs.create()
                        .owner(experimentViewController)
                        .message("An error occurred while generating the experiment-info JSON.")
                        .masthead("Error")
                        .title("Error")
                        .showException(ex));
                return null;
            }

            LOG.debug("      experiment info JSON is "+fileContent.length+" bytes long");

            try (FileOutputStream os = new FileOutputStream(file)) {
                os.write(fileContent);
                os.flush();
                os.close();
            } catch (final IOException ex) {
                LOG.error("Error while saving experiment info JSON in \"" + file.getAbsolutePath() + "\"", ex);
                Platform.runLater(() -> JFDialogs.create()
                        .owner(experimentViewController)
                        .message("An error occurred while saving the experiment-info JSON.")
                        .masthead("Error")
                        .title("Error")
                        .showException(ex));
                return null;
            }
            updateMessage("Successfully saved experiment-info JSON in \"" + file.getAbsolutePath() + "\"");

            return null;
        }
    }

}
